package modelage;

import java.awt.event.ActionEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;


/**
 * This program solves the problem of extracting the diagenetic and detrital 
 * ages and of the 40Kdetrital/40Kdiagenetic ratio from K-Ar dates 
 * of clay fraction. 
 * 
 * This class defines the outlook of the program and provides primary methods  
 * 
 * For more references see Szczerba & rodo (Clays and Clay Minerals, 57, 4656, 2009).
 * 
 * In case of any questions, any problems, or especially if you find any errors in the program 
 * please contact me:
 * 
 * Marek Szczerba
 * Institute of Geological Sciences Polish Academy of Sciences
 * Senacka 1 31-002 Krakw, Poland
 * e-mail: ndszczer@cyf-kr.edu.pl
 * 
 * @author  ndszczer@cyf-kr.edu.pl
 */

public class MODELAGE extends javax.swing.JFrame {  	
	private static final long serialVersionUID = 1L;
	/**
	 * @serial 1.0
	 */
	
	private int noOfInputValues;
	
    private double[] initialArKValues = new double[6];
	private double[] initialWtValues = new double[6];
	private static double[] initialArKValuesClone; 
	private static double[] initialWtValuesClone; 
	private double[] initialArKValuesCloneSort;

	public static boolean systematicAgeKnown = false;
	public static double systematicAge = 0;
	public static double systematicError = 0;

	private static double randomError = 0;
	private int noOfCalculations = 0;
	
    private SwingWorker worker;
    private int beginEvolution = 0;

    
    
    static javax.swing.JLabel ArKDetrLabel;
    static javax.swing.JLabel ArKDiagLabel;
    static javax.swing.JLabel wtDetrLabel;
    static javax.swing.JLabel KDetrLabel;
    static javax.swing.JLabel KDiagLabel;
    static javax.swing.JLabel KPerKLabel;
    static javax.swing.JButton dataPlotButton;
    static javax.swing.JLabel detrAgeLabel;
    static javax.swing.JLabel diagAgeLabel;
    static javax.swing.JLabel fitLabel;
    static javax.swing.JPanel jPanel2;   
    static javax.swing.JLabel progressLabel;
    
    private static javax.swing.JProgressBar progressBar;
    private static javax.swing.JButton saveButton;
    private static javax.swing.JButton startButton;
 
    private javax.swing.JLabel detrAgeSuspLabel;
    private javax.swing.JTextField systematicAgeTextField;
    private javax.swing.JTextField randomTextField;
    private javax.swing.JTextField systematicTextField;
    private javax.swing.JTextField ark2;
    private javax.swing.JTextField ark3;
    private javax.swing.JCheckBox ageDetrCheckBox;
    private javax.swing.JLabel ageDetrHighLabel;
    private javax.swing.JTextField ageDetrHighTextField;
    private javax.swing.JTextField ageDetrLowTextField;
    private javax.swing.JCheckBox ageDiagCheckBox;
    private javax.swing.JTextField ageDiagHighTextField;
    private javax.swing.JLabel ageDiagLabel;
    private javax.swing.JTextField ageDiagLowTextField;
    private javax.swing.JLabel ageLabel;
    private javax.swing.JTextField ark1;
    private javax.swing.JTextField ark4;
    private javax.swing.JTextField ark5;
    private javax.swing.JTextField ark6;  
    private javax.swing.JPanel jPanel1;   
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JCheckBox kDetrCheckBox;
    private javax.swing.JTextField kDetrHighTextField;
    private javax.swing.JLabel kDetrLabel;
    private javax.swing.JTextField kDetrLowTextField;
    private javax.swing.JCheckBox kDiagCheckBox;
    private javax.swing.JTextField kDiagHighTextField;
    private javax.swing.JLabel kDiagLabel;
    private javax.swing.JTextField kDiagLowTextField;
    private javax.swing.JLabel noCalcLabel;
    private javax.swing.JTextField noCalcTextField;
    private javax.swing.JLabel randomLabel;
    private javax.swing.JButton stopButton;
    private javax.swing.JLabel systematicLabel;
    private javax.swing.JTextField wt1;
    private javax.swing.JTextField wt2;
    private javax.swing.JTextField wt3;
    private javax.swing.JTextField wt4;
    private javax.swing.JTextField wt5;
    private javax.swing.JTextField wt6;
    private javax.swing.JFileChooser saveDiffractogramChooser;

	/**
	 * constructor
	 */
	public MODELAGE() {
        initComponents();
    }

	/**
	 * Swing element setting on the screen
	 */
	private void initComponents() {
    	
	//Setting System appearance 
		try{
   		 	UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
   	 	}
   	 	catch(Exception e){
   	 		throw new RuntimeException(e);
   	 	}

   	 	jPanel1 = new javax.swing.JPanel();
   	 	jPanel2 = new javax.swing.JPanel();
   	 	jPanel3 = new javax.swing.JPanel();
   	 	jPanel4 = new javax.swing.JPanel();
   	 	jPanel5 = new javax.swing.JPanel();
   	 	jPanel6 = new javax.swing.JPanel();
   	 	jPanel7 = new javax.swing.JPanel();
   	 	jPanel8 = new javax.swing.JPanel();
   	 	jPanel9 = new javax.swing.JPanel();
        
   	 	wt1 = new javax.swing.JTextField();
   	 	wt2 = new javax.swing.JTextField();
   	 	wt3 = new javax.swing.JTextField();
   	 	wt4 = new javax.swing.JTextField();   
   	 	wt5 = new javax.swing.JTextField();
   	 	wt6 = new javax.swing.JTextField();
   	 	ark1 = new javax.swing.JTextField();
   	 	ark2 = new javax.swing.JTextField();
   	 	ark3 = new javax.swing.JTextField();
   	 	ark4 = new javax.swing.JTextField();
   	 	ark5 = new javax.swing.JTextField();
   	 	ark6 = new javax.swing.JTextField();
   	 	wtDetrLabel = new javax.swing.JLabel();
   	 	ageLabel = new javax.swing.JLabel();    

   	 	kDiagLowTextField = new javax.swing.JTextField();
   	 	kDiagHighTextField = new javax.swing.JTextField();
   	 	kDiagCheckBox = new javax.swing.JCheckBox();
   	 	kDiagLabel = new javax.swing.JLabel();   

   	 	kDetrLowTextField = new javax.swing.JTextField();
   	 	kDetrHighTextField = new javax.swing.JTextField();
   	 	kDetrCheckBox = new javax.swing.JCheckBox();
   	 	kDetrLabel = new javax.swing.JLabel();
     
   	 	ageDiagLowTextField = new javax.swing.JTextField();
   	 	noCalcTextField = new javax.swing.JTextField();
   	 	ageDiagHighTextField = new javax.swing.JTextField();
   	 	ageDiagLabel = new javax.swing.JLabel();
   	 	ageDiagCheckBox = new javax.swing.JCheckBox();
     
   	 	ageDetrLowTextField = new javax.swing.JTextField();
   	 	ageDetrHighLabel = new javax.swing.JLabel();
   	 	ageDetrHighTextField = new javax.swing.JTextField();
   	 	ageDetrCheckBox = new javax.swing.JCheckBox();

   	 	progressLabel = new javax.swing.JLabel();
   	 	progressBar = new javax.swing.JProgressBar();
   	 	noCalcLabel = new javax.swing.JLabel();
   	 	startButton = new javax.swing.JButton();
   	 	stopButton = new javax.swing.JButton();
     
   	 	ArKDiagLabel = new javax.swing.JLabel();
   	 	diagAgeLabel = new javax.swing.JLabel();
   	 	detrAgeLabel = new javax.swing.JLabel();
   	 	KPerKLabel = new javax.swing.JLabel();
   	 	KDetrLabel = new javax.swing.JLabel();
   	 	KDiagLabel = new javax.swing.JLabel();
   	 	fitLabel = new javax.swing.JLabel();
   	 	saveButton = new javax.swing.JButton();
   	 	ArKDetrLabel = new javax.swing.JLabel();
     
   	 	randomLabel = new javax.swing.JLabel();
   	 	systematicLabel = new javax.swing.JLabel();
   	 	randomTextField = new javax.swing.JTextField();
   	 	systematicTextField = new javax.swing.JTextField();
   	 	systematicAgeTextField = new javax.swing.JTextField();
   	 	detrAgeSuspLabel = new javax.swing.JLabel();
     
   	 	ResultsPanel jPanel2 = new ResultsPanel();
	 	stopButton.setEnabled(false);
	 	
	    progressLabel.setText("progress:");
	    noCalcLabel.setText("no of calculations");
	    stopButton.setText("stop");
	    startButton.setText("start");
	    ArKDiagLabel.setText("40Ar/40K diag");
	    diagAgeLabel.setText("diagenetic age");
	    detrAgeLabel.setText("detritial age");
	    KPerKLabel.setText("K detr / K diag");
	    KDetrLabel.setText("K detr");
	    KDiagLabel.setText("K diag");
	    fitLabel.setText("fit");
	    saveButton.setText("save results");
	    ArKDetrLabel.setText("40Ar/40K detr");
	    
   	//some initial values   	 	
   	 	wt1.setText("31");
   	 	wt2.setText("44");
   	 	wt3.setText("65");
   	 	noCalcTextField.setText("1");
   	 	kDetrLowTextField.setText("7");
   	 	kDiagLowTextField.setText("5");
   	 	kDetrHighTextField.setText("7");
   	 	kDiagHighTextField.setText("5");
   	 	ark1.setText("237");
   	 	ark2.setText("255");
   	 	ark3.setText("284");
          
    //listeners   
   	 	startButton.addActionListener(new java.awt.event.ActionListener() {
   	 		public void actionPerformed(final java.awt.event.ActionEvent evt) {           	
   	 			startButtonActionPerformed(evt);               
   	 		}
   	 	});
   	 	
   	 	stopButton.addActionListener(new java.awt.event.ActionListener() {
   	 		public void actionPerformed(final java.awt.event.ActionEvent evt) {           	
   	 			stopButtonActionPerformed(evt);               
   	 		}
   	 	});
    
   	 	saveButton.addActionListener(new java.awt.event.ActionListener() {
   	 		public void actionPerformed(final java.awt.event.ActionEvent evt) {           	
   	 			saveButtonButtonActionPerformed(evt);               
   	 		}
   	 	});
     
   	 	setTitle("MODELAGE 1.0");
   	 	setAlwaysOnTop(false);   
   	 	setResizable(false);
   	 	setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
   	 	setLocation(150,50);
   	 	setSize(new java.awt.Dimension(500, 350));
   	 	setMaximumSize(new java.awt.Dimension(50, 35));
   	 	setMinimumSize(null);
     
     jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "input variables"));
     wtDetrLabel.setText("%wt detr");
     ageLabel.setText("age (Ma)");
     jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(153, 153, 255)), "%K diag"));
     kDiagCheckBox.setSelected(true);
     kDiagCheckBox.setText("fixed");
     kDiagCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
     kDiagCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
     kDiagLabel.setText(" -");

     javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
     jPanel4.setLayout(jPanel4Layout);
     jPanel4Layout.setHorizontalGroup(
         jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel4Layout.createSequentialGroup()
             .addContainerGap()
             .addComponent(kDiagLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(kDiagLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
             .addComponent(kDiagHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(kDiagCheckBox)
             .addContainerGap())
     );
     jPanel4Layout.setVerticalGroup(
         jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel4Layout.createSequentialGroup()
             .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(kDiagLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(kDiagCheckBox)
                 .addComponent(kDiagLabel)
                 .addComponent(kDiagHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
     );

     jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(153, 153, 255)), "%K detr"));

     kDetrCheckBox.setText("fixed");
     kDetrCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
     kDetrCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

     kDetrLabel.setText(" -");

     javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
     jPanel5.setLayout(jPanel5Layout);
     jPanel5Layout.setHorizontalGroup(
         jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel5Layout.createSequentialGroup()
             .addContainerGap()
             .addComponent(kDetrLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(kDetrLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
             .addComponent(kDetrHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(kDetrCheckBox)
             .addContainerGap())
     );
     jPanel5Layout.setVerticalGroup(
         jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel5Layout.createSequentialGroup()
             .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(kDetrLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(kDetrCheckBox)
                 .addComponent(kDetrLabel)
                 .addComponent(kDetrHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
     );

     jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 153, 51)), "diagenetic age"));

     ageDiagLabel.setText(" -");

     ageDiagCheckBox.setText("fixed");
     ageDiagCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
     ageDiagCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

     javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
     jPanel6.setLayout(jPanel6Layout);
     jPanel6Layout.setHorizontalGroup(
         jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel6Layout.createSequentialGroup()
             .addContainerGap()
             .addComponent(ageDiagLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(ageDiagLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
             .addComponent(ageDiagHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(ageDiagCheckBox)
             .addContainerGap())
     );
     jPanel6Layout.setVerticalGroup(
         jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel6Layout.createSequentialGroup()
             .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(ageDiagLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ageDiagLabel)
                 .addComponent(ageDiagCheckBox)
                 .addComponent(ageDiagHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
     );

     jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 153, 51)), "detrital age"));

     ageDetrHighLabel.setText(" -");
     ageDetrCheckBox.setText("fixed");
     ageDetrCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
     ageDetrCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

     javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
     jPanel7.setLayout(jPanel7Layout);
     jPanel7Layout.setHorizontalGroup(
         jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel7Layout.createSequentialGroup()
             .addContainerGap()
             .addComponent(ageDetrLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(ageDetrHighLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 5, Short.MAX_VALUE)
             .addComponent(ageDetrHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(ageDetrCheckBox)
             .addContainerGap())
     );
     jPanel7Layout.setVerticalGroup(
         jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel7Layout.createSequentialGroup()
             .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(ageDetrLowTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ageDetrHighLabel)
                 .addComponent(ageDetrCheckBox)
                 .addComponent(ageDetrHighTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
     );

     javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
     jPanel1.setLayout(jPanel1Layout);
     jPanel1Layout.setHorizontalGroup(
         jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel1Layout.createSequentialGroup()
             .addContainerGap()
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                 .addGroup(jPanel1Layout.createSequentialGroup()
                     .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                         .addComponent(wtDetrLabel)
                         .addComponent(wt3)
                         .addComponent(wt1)
                         .addComponent(wt2, javax.swing.GroupLayout.DEFAULT_SIZE, 64, Short.MAX_VALUE)
                         .addComponent(wt4)
                         .addComponent(wt5)
                         .addComponent(wt6))
                     .addGap(10, 10, 10)
                     .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                         .addComponent(ageLabel)
                         .addComponent(ark2)
                         .addComponent(ark3)
                         .addComponent(ark1, javax.swing.GroupLayout.DEFAULT_SIZE, 66, Short.MAX_VALUE)
                         .addComponent(ark4)
                         .addComponent(ark5)
                         .addComponent(ark6)))
                 .addComponent(jPanel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addComponent(jPanel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addComponent(jPanel7, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addContainerGap())
     );
     jPanel1Layout.setVerticalGroup(
         jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel1Layout.createSequentialGroup()
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wtDetrLabel)
                 .addComponent(ageLabel))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wt1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ark1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wt2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ark2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wt3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ark3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wt4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ark4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wt5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ark5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(wt6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(ark6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(jPanel7, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addContainerGap(22, Short.MAX_VALUE))
     );

     jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "40Ar / 40K data plot"));
     javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
     jPanel2.setLayout(jPanel2Layout);
     jPanel2Layout.setHorizontalGroup(
         jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGap(0, 458, Short.MAX_VALUE)
     );
     jPanel2Layout.setVerticalGroup(
         jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGap(0, 436, Short.MAX_VALUE)
     );

     jPanel8.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "GA"));

     javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
     jPanel8.setLayout(jPanel8Layout);
     jPanel8Layout.setHorizontalGroup(
         jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel8Layout.createSequentialGroup()
             .addContainerGap()
             .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                 .addGroup(jPanel8Layout.createSequentialGroup()
                     .addComponent(noCalcLabel)
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 14, Short.MAX_VALUE)
                     .addComponent(noCalcTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 54, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel8Layout.createSequentialGroup()
                     .addComponent(startButton, javax.swing.GroupLayout.PREFERRED_SIZE, 73, javax.swing.GroupLayout.PREFERRED_SIZE)
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addComponent(stopButton, javax.swing.GroupLayout.PREFERRED_SIZE, 72, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addComponent(progressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 151, Short.MAX_VALUE)
                 .addComponent(progressLabel))
             .addContainerGap())
     );
     jPanel8Layout.setVerticalGroup(
         jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel8Layout.createSequentialGroup()
             .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(noCalcLabel)
                 .addComponent(noCalcTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(progressLabel)
             .addGap(1, 1, 1)
             .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(stopButton)
                 .addComponent(startButton))
             .addContainerGap())
     );

     jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "results"));

     javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
     jPanel3.setLayout(jPanel3Layout);
     jPanel3Layout.setHorizontalGroup(
         jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel3Layout.createSequentialGroup()
             .addContainerGap()
             .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                 .addGroup(jPanel3Layout.createSequentialGroup()
                     .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                         .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel3Layout.createSequentialGroup()
                             .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                 .addComponent(KPerKLabel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 70, Short.MAX_VALUE)
                                 .addComponent(detrAgeLabel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 70, Short.MAX_VALUE)
                                 .addComponent(diagAgeLabel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                 .addComponent(ArKDiagLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 70, Short.MAX_VALUE))
                             .addGap(226, 226, 226))
                         .addComponent(KDetrLabel)
                         .addComponent(KDiagLabel)
                         .addComponent(fitLabel))
                     .addGap(0, 0, 0))
                 .addGroup(jPanel3Layout.createSequentialGroup()
                     .addComponent(ArKDetrLabel)
                     .addContainerGap(93, Short.MAX_VALUE))
                 .addGroup(jPanel3Layout.createSequentialGroup()
                     .addComponent(saveButton)
                     .addContainerGap(70, Short.MAX_VALUE))))
     );
     jPanel3Layout.setVerticalGroup(
         jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel3Layout.createSequentialGroup()
             .addComponent(ArKDiagLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(ArKDetrLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(diagAgeLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(detrAgeLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(KPerKLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(KDiagLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(KDetrLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(fitLabel)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(saveButton)
             .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
     );

     jPanel9.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "errors"));
     randomLabel.setText("random");

     systematicLabel.setText("systematic");

     detrAgeSuspLabel.setText("1M detritial suspected age");

     javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
     jPanel9.setLayout(jPanel9Layout);
     jPanel9Layout.setHorizontalGroup(
         jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel9Layout.createSequentialGroup()
             .addContainerGap()
             .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                 .addComponent(randomLabel)
                 .addComponent(systematicLabel)
                 .addComponent(detrAgeSuspLabel))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 25, Short.MAX_VALUE)
             .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                 .addComponent(randomTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 54, Short.MAX_VALUE)
                 .addComponent(systematicTextField)
                 .addComponent(systematicAgeTextField))
             .addContainerGap())
     );
     jPanel9Layout.setVerticalGroup(
         jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(jPanel9Layout.createSequentialGroup()
             .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(randomLabel)
                 .addComponent(randomTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(systematicLabel)
                 .addComponent(systematicTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(detrAgeSuspLabel)
                 .addComponent(systematicAgeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addContainerGap(11, Short.MAX_VALUE))
     );

     javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
     getContentPane().setLayout(layout);
     layout.setHorizontalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
             .addContainerGap()
             .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
             .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                 .addComponent(jPanel8, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addComponent(jPanel9, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addComponent(jPanel3, 0, 183, Short.MAX_VALUE))
             .addContainerGap(26, Short.MAX_VALUE))
     );
     layout.setVerticalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
             .addContainerGap()
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                 .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 462, Short.MAX_VALUE)
                 .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                 .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                     .addComponent(jPanel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                     .addComponent(jPanel8, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                     .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
             .addContainerGap(23, Short.MAX_VALUE))
     );
     pack();
	}///=======================================================================                                

    
	
	
	
//==============================  listeners  ==================================	
    /**
     * "start" button action performed --> reading input values and genetic algorithm initialization    
     */
	private void startButtonActionPerformed(final java.awt.event.ActionEvent evt) {
    //resetting initial value	
    	ResultsPanel.geneticPerformed = 0; 

    //error windows appearances------------------------------------------------
		class numberFormatExceptionDialog extends JFrame{		
			numberFormatExceptionDialog() { 
    			super(); 
    			JOptionPane.showMessageDialog(
    				jPanel1, "Inappropriate values were given. \n" +
 			     		"   Only numbers are accepted.",
 			     		"Inappropriate values warning",
 			    JOptionPane.INFORMATION_MESSAGE);
    		}
    	}
		
		class numberFormatErrorExceptionDialog extends JFrame{		
    		numberFormatErrorExceptionDialog() { 
    			super(); 
    			JOptionPane.showMessageDialog(
    				jPanel9, "Inappropriate values for errors were given. \n" +
 			     		"       Only numbers are accepted.",
 			     		"Inappropriate values warning",
 			    JOptionPane.INFORMATION_MESSAGE);
    		}
    	}

		class numberFormatNoExceptionDialog extends JFrame{		
    		numberFormatNoExceptionDialog() { 
    			super(); 
    			JOptionPane.showMessageDialog(
    				jPanel8, "Inappropriate value for no of calculations were given. \n" +
 			     		"                   Only numbers are accepted.",
 			     		"Inappropriate values warning",
 			    JOptionPane.INFORMATION_MESSAGE);
    		}
    	}

		class inappropriateValuesDialog extends JFrame{
			inappropriateValuesDialog() { 
				super(); 
				JOptionPane.showMessageDialog(
					jPanel1, "    Inappropriate values for %wt were given. \n" +
 			     		"Accepted only higher than 0 and lower than 100.",
 			     		"Inappropriate values warning",
 			    JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
		class inappropriateNoValuesDialog extends JFrame{
			inappropriateNoValuesDialog() { 
				super(); 
				JOptionPane.showMessageDialog(
					jPanel8, "Inappropriate values for no of calculations were given. \n" +
 			     		"                     Accepted only integer values.",
 			     		"Inappropriate values warning",
 			    JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
		boolean correctValues = true; //flag which determines if read values are correct
	
	//reading %wt and age values of particular fractions and checking if correct	
		try {
     		if(wt1.getText().compareTo("")!=0){
     			noOfInputValues = 1;
     			initialArKValues[0] = ArKCalculate((double)((Double.parseDouble(ark1.getText()))));
         		initialWtValues[0] = (double)((Double.parseDouble(wt1.getText()))); 
         	   
         		if(wt2.getText().compareTo("")!=0){
         			noOfInputValues = 2;
            		initialArKValues[1] = ArKCalculate((double)((Double.parseDouble(ark2.getText()))));
            		initialWtValues[1] = (double)((Double.parseDouble(wt2.getText())));    
         		
            		if(wt3.getText().compareTo("")!=0){
            			noOfInputValues = 3;
               			initialArKValues[2] = ArKCalculate((double)((Double.parseDouble(ark3.getText()))));
               			initialWtValues[2] = (double)((Double.parseDouble(wt3.getText())));
            			
               			if(wt4.getText().compareTo("")!=0){
              				noOfInputValues = 4;
                   			initialArKValues[3] = ArKCalculate((double)((Double.parseDouble(ark4.getText()))));
                   			initialWtValues[3] = (double)((Double.parseDouble(wt4.getText())));
                     			
                   			if(wt5.getText().compareTo("")!=0){
                   				noOfInputValues = 5;
                       			initialArKValues[4] = ArKCalculate((double)((Double.parseDouble(ark5.getText()))));
                       			initialWtValues[4] = (double)((Double.parseDouble(wt5.getText())));
                     			
                      			if(wt6.getText().compareTo("")!=0){
                      				noOfInputValues = 6;
                           			initialArKValues[5] = ArKCalculate((double)((Double.parseDouble(ark6.getText()))));
                           			initialWtValues[5] = (double)((Double.parseDouble(wt6.getText())));
                       			}
                   			}
               			}
            		}
         		}
     		}
     		
     		MODELAGEMinimizing.setNoOfInputValues(noOfInputValues);
     		
     		initialArKValuesClone = new double[noOfInputValues];//table formed in order to sort it and subsequently draw diagram
     		initialArKValuesCloneSort = new double[noOfInputValues];//sorted table
     		initialWtValuesClone = new double[noOfInputValues];    		
     		for(int i=0; i<noOfInputValues; i++){
    			initialArKValuesClone[i] = initialArKValues[i];
    			initialArKValuesCloneSort[i] = initialArKValues[i];
    			initialWtValuesClone[i] = initialWtValues[i];
 			}
     		Arrays.sort(initialArKValuesCloneSort);

     	//checking if %wt values are in appropriate ranges	
     		boolean inappropriateRanges = false;//flag which determines if %wt values are in appropriate ranges    			
     	    for(int i=0; i<noOfInputValues; i++){
     	    	if (initialWtValues[i] < 0 || initialWtValues[i] > 100)	{
     	    		inappropriateRanges = true;
     	    	}
     	    }    	    
     	    if (inappropriateRanges == true){
 	    		inappropriateValuesDialog inappropriateValuesDialog = new inappropriateValuesDialog();
 	     		correctValues = false;
 	    	}
     	
     	//setting apropriate values of GA if particular ranges were fixed    
     	    MODELAGEMinimizing.unfixAll();//unfix all properties     			
     		if (kDiagCheckBox.isSelected()){
     			MODELAGEMinimizing.fixKDiag
     			((Double.parseDouble(kDiagLowTextField.getText())),(Double.parseDouble(kDiagHighTextField.getText())));
     		}
     		if (kDetrCheckBox.isSelected())	{
     			MODELAGEMinimizing.fixKDetr
     			((Double.parseDouble(kDetrLowTextField.getText())),(Double.parseDouble(kDetrHighTextField.getText())));
     		}	
     		if (ageDetrCheckBox.isSelected())	{
     			MODELAGEMinimizing.fixAgeDetr
     			((Double.parseDouble(ageDetrLowTextField.getText())),(Double.parseDouble(ageDetrHighTextField.getText())));
     		}	
     		if (ageDiagCheckBox.isSelected()){
     			MODELAGEMinimizing.fixAgeDiag
     			((Double.parseDouble(ageDiagLowTextField.getText())),(Double.parseDouble(ageDiagHighTextField.getText())));
     		}

     	//calcuation of values that will be further used to constrain expected values of genes in GA
     		double ageSum = 0;
     		for(int i=0; i<noOfInputValues; i++){
     			ageSum += initialArKValuesClone[i];
     		}    				
     		MODELAGEMinimizing.ArKGeneRangeMiddle = ageSum/noOfInputValues;
     			
     	}     	
    //throw dialog box that informs about inappropriate values e.g. empty box or character instead of float value
     	catch ( NumberFormatException e) {
     		numberFormatExceptionDialog numberFormatExceptionDialog = new numberFormatExceptionDialog();
     		correctValues = false;
     	}
     	
    //reading error/uncertainities values
     	try {
     		systematicAgeKnown = false;
     		systematicAge = 0;
     		systematicError = 0;
     		randomError = 0;
     		if(systematicTextField.getText().compareTo("")!=0){
     			systematicError = (double)((Double.parseDouble(systematicTextField.getText())));
     		}
     		if(randomTextField.getText().compareTo("")!=0){
     			randomError = (double)((Double.parseDouble(randomTextField.getText())));
     		}	
     		if(systematicAgeTextField.getText().compareTo("")!=0){
     			systematicAge = (double)((Double.parseDouble(systematicAgeTextField.getText())));
     			systematicAgeKnown = true; //other way of including systematic error
     		}
     	} 
     	catch ( NumberFormatException e) {
         	numberFormatErrorExceptionDialog numberFormatErrorExceptionDialog = new numberFormatErrorExceptionDialog();
         	correctValues = false;
        }
     
    //reading no of calculations
     	try {
     		double noOfCalculationsDouble = (double)((Double.parseDouble(noCalcTextField.getText())));
     		if (noOfCalculationsDouble != (int)noOfCalculationsDouble){
 	    		inappropriateNoValuesDialog inappropriateNoValuesDialog = new inappropriateNoValuesDialog();
 	     		correctValues = false;
     		} else{
     			noOfCalculations = (int)noOfCalculationsDouble;
     		}
     	} 
     	catch ( NumberFormatException e) {
         	numberFormatNoExceptionDialog numberFormatNoExceptionDialog = new numberFormatNoExceptionDialog();
         	correctValues = false;
        }
     		
    //begin evolution----------------------------------------------------------
    	if (correctValues == true){
        	startButton.setEnabled(false);
    		stopButton.setEnabled(true);

    	//initialization of genetic alorithms	
    		MODELAGEMinimizing.beginEvolution(initialArKValuesClone, initialWtValuesClone, noOfInputValues, randomError, systematicError, systematicAge, noOfCalculations); 
			
    	//worker operates on threads	
    		worker = new SwingWorker() {
    		//blot out of construct() method in SwingWorker - operates time demanding GA calculations
    			public Object construct() {
    				updateStatus(0);
    				if(beginEvolution!=1){
    					return MODELAGEMinimizing.Evolution();//initialization of time demanding GA calculations
    				}
    				return null;
    			}
    			
    		//blot out of finished() method in SwingWorker - presents results
            	public void finished() {
                	startButton.setEnabled(true);
            		stopButton.setEnabled(false);
          		
                //points drawing
            		final double middle = (initialArKValuesCloneSort[0]+initialArKValuesCloneSort[initialArKValuesCloneSort.length-1])/2;
            		ResultsPanel.middle = (int)middle;
            		progressLabel.setText("progress:");
            		
            	  //ArK values are modified	
            		if (systematicAgeKnown == true){
            			double[] ArKValuesModified = new double[initialArKValuesClone.length];
            			for(int i = 0; i<initialArKValuesClone.length; i++){
            				ArKValuesModified[i] = ArKCorrectToDetritialComponent (initialArKValuesClone[i], systematicError, systematicAge);;
            			}
            		}
            		
            		ResultsPanel.setInitialArKValues(initialArKValuesClone);
            		repaint();
            	}
    		};
        	worker.start();
    	}
    }
		
    /**
     * "stop" button action performed --> interruption of worker
     */	
	private void stopButtonActionPerformed (final java.awt.event.ActionEvent evt) {
    	updateStatus(0);
    	stopButton.setEnabled(false);
        worker.interrupt();
        startButton.setEnabled(true);
    } 	

    /**
     * "save results" button action performed --> Saves the results to a *.txt file
     */   
 	private void saveButtonButtonActionPerformed(ActionEvent evt) {  
 	//Set up the file chooser.
 		saveDiffractogramChooser = new JFileChooser();
 	//Add a custom file filter and disable the default
	    saveDiffractogramChooser.addChoosableFileFilter(new TxtFileFilter());  
	//show it	
	    int returnVal = saveDiffractogramChooser.showSaveDialog(ArKDetrLabel);	
	//Process the results.
	    if (returnVal == JFileChooser.APPROVE_OPTION){
	        File file = saveDiffractogramChooser.getSelectedFile();
	        	
	        if (file.exists()){
	        	JOptionPane.showConfirmDialog(ArKDetrLabel, "Overwrite existing file?", 
	        	  "Confirm Overwrite", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
	        }
	        	
	    //creating output string  		        	
	        StringBuffer sb = new StringBuffer();
	        
	    //diagenetic age part	
	        sb.append("Diagenetic age : \n");
	        double ageDiagFinalSum = 0;
	        double ageDiagAverage = 0;
	        for(int z = 0; z < MODELAGEMinimizing.ArPerKDiagFinal.length; z++){
	        	ageDiagFinalSum += ageCalculate(MODELAGEMinimizing.ArPerKDiagFinal[z]);
	        	sb.append(ageCalculate(MODELAGEMinimizing.ArPerKDiagFinal[z])).append("\n");
	        }
        	ageDiagAverage = ageDiagFinalSum/MODELAGEMinimizing.ArPerKDiagFinal.length;
        	sb.append("Average: ").append(ageDiagAverage).append("\n");
	        double ageDiagDifferenceSum = 0;
	        for(int z = 0; z < MODELAGEMinimizing.ArPerKDiagFinal.length; z++){
	        	ageDiagDifferenceSum += Math.pow((ageCalculate(MODELAGEMinimizing.ArPerKDiagFinal[z]) - ageDiagAverage), 2);
	        }
        	double stdDeviationArPerKDiag = 0;
	        if (MODELAGEMinimizing.ArPerKDiagFinal.length > 1){
	        	stdDeviationArPerKDiag = Math.sqrt((1.0/(double)(MODELAGEMinimizing.ArPerKDiagFinal.length-1))*ageDiagDifferenceSum);
	        	sb.append("Standard_deviation: ").append(stdDeviationArPerKDiag).append("\n\n");
	        }
	        
	    //detrital age part	
	        sb.append("Detritial age : \n");
	        double ageDetrFinalSum = 0;
	        double ageDetrAverage = 0;
	        for(int z = 0; z < MODELAGEMinimizing.ArPerKDetrFinal.length; z++){
	        	ageDetrFinalSum += ageCalculate(MODELAGEMinimizing.ArPerKDetrFinal[z]);
	        	sb.append(ageCalculate(MODELAGEMinimizing.ArPerKDetrFinal[z])).append("\n");
	        }
        	ageDetrAverage = ageDetrFinalSum/MODELAGEMinimizing.ArPerKDetrFinal.length;
        	sb.append("Average: ").append(ageDetrAverage).append("\n");
	       	double ageDetrDifferenceSum = 0;
	       	for(int z = 0; z < MODELAGEMinimizing.ArPerKDetrFinal.length; z++){
	       		ageDetrDifferenceSum += Math.pow((ageCalculate(MODELAGEMinimizing.ArPerKDetrFinal[z]) - ageDetrAverage), 2);
	       	}
        	double stdDeviationArPerKDetr = 0;
	       	if (MODELAGEMinimizing.ArPerKDetrFinal.length > 1){
	       		stdDeviationArPerKDetr = Math.sqrt((1.0/(double)(MODELAGEMinimizing.ArPerKDetrFinal.length-1))*ageDetrDifferenceSum);
	       		sb.append("Standard_deviation: ").append(stdDeviationArPerKDetr).append("\n\n");
	       	}
	       	
	    //K_detritial/K_diagenetic part		
	        sb.append("K_detritial/K_diagenetic: \n");
	        double KdetrKdiagFinalSum = 0;
	        double KdetrKdiagAverage = 0;
	        for(int z = 0; z < MODELAGEMinimizing.KdetrPerKdiagFinal.length; z++){
	       		KdetrKdiagFinalSum += MODELAGEMinimizing.KdetrPerKdiagFinal[z];
	       		sb.append(MODELAGEMinimizing.KdetrPerKdiagFinal[z]).append("\n");
	       	}
	       	KdetrKdiagAverage = KdetrKdiagFinalSum/MODELAGEMinimizing.KdetrPerKdiagFinal.length;
        	sb.append("Average: ").append(KdetrKdiagAverage).append("\n");
	       	double KdetrKdiagDifferenceSum = 0;
	       	for(int z = 0; z < MODELAGEMinimizing.KdetrPerKdiagFinal.length; z++){
	        	KdetrKdiagDifferenceSum += Math.pow((MODELAGEMinimizing.KdetrPerKdiagFinal[z] - KdetrKdiagAverage), 2);
	        }
        	double stdDeviationKdetrKdiag = 0;
	        if (MODELAGEMinimizing.KdetrPerKdiagFinal.length > 1){
	        	stdDeviationKdetrKdiag = Math.sqrt((1.0/(double)(MODELAGEMinimizing.KdetrPerKdiagFinal.length-1))*KdetrKdiagDifferenceSum);
	       		sb.append("Standard_deviation: ").append(stdDeviationKdetrKdiag).append("\n\n");
	       	}

	    //writes the file    
	        String output = sb.toString();	         	
	        writeFile(file, output);

	    //Reset the file chooser for the next time it's shown.
	        saveDiffractogramChooser.setSelectedFile(null);	
	    }
 	}	

 	
 	
 	
 	
//============================ backup methods  ================================ 
 	
    /**
     * method that operates on progress bar
     */
    static void updateStatus(final int i) {
        Runnable doSetProgressBarValue = new Runnable() {
            public void run() {
                progressBar.setValue(i);
            }
        };
        SwingUtilities.invokeLater(doSetProgressBarValue);
    }
 	
    /**
     * calculates age (in Ma) from 40Ar/40K
     */   
    private static double ageCalculate(double KArRelation) {
    	return Math.log((1.4263 * KArRelation/100) + 1) / 5.543 * 10000; //returns age in Ma
    }

    /**
     * calculates 40Ar/40K from age (in Ma)
     */
    private static double ArKCalculate(double age) {
    	return (Math.exp(age/10000*5.543) - 1)/1.4263 * 100;
    }
    
    /**
     * corrects 40Ar/40K values if 1Md detrital age is known 
     */
    private static double ArKCorrectToDetritialComponent (double ArKTotal, double systematicError, double systematicAge){
    	return (100 * ArKTotal - systematicError * ArKCalculate(systematicAge))/(100 - systematicError);
    }
    
 	/**
 	 * prints out the dataString to a file
 	 */ 
 	private static boolean writeFile (File file, String dataString){
 	    try{
 	    	PrintWriter out = 
 	    		new PrintWriter (new BufferedWriter (new FileWriter(file)));
 	    	out.print (dataString);
 	    	out.flush ();
 	    	out.close ();
 	    } catch (IOException e){
 	    	return false;
 	    }
 	    return true;
 	}
    
 	
 	
 	
 	
//==============================  MAIN  ======================================= 	
    /**
     * main method - displays the MODELAGE window
     */
    public static void main(final String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new MODELAGE().setVisible(true);
            }
        });
    }    
}///:~









